import React from 'react';
import config from 'config';
import request from '../../../utils/request';
import { formatTime, getHashCode } from '../../../utils/format';
import styles from './index.module.less';

import pdfImg from '../../../assets/pdf.png';
import docImg from '../../../assets/doc.png';
import xlsImg from '../../../assets/xls.png';
import pptImg from '../../../assets/ppt.svg';

interface IFileInfo {
  [propertys: string]: any,
  id: string,
}

interface IDocComponent {
  loadSuccess: () => void,
  style?: any
}

const ImgObject = {
  w: docImg,
  s: xlsImg,
  p: pptImg,
  f: pdfImg
}
const wpsFileId = config.wpsFileId;

export default class DocComponent extends React.Component<IDocComponent> {

  wsServer = null;
  socketTimer = null;
  reconnectTimer = null;
  wpsInstantce = null;
  time = getHashCode();
  state = {
    user: {} as any,
    wsConfig: {} as any,
    messageList: [],
    fileInfo: {} as IFileInfo
  }

  componentDidMount() {
    this.getFielList();
    // @插件通信，接受查询信息
    window.addEventListener("message", this.messageListener, false);
  }

  componentWillUnmount() {
    this.wsServer && this.wsServer.close();
    this.socketTimer && clearInterval(this.socketTimer);
    this.reconnectTimer && clearTimeout(this.reconnectTimer);
    window.removeEventListener("message", this.messageListener)
  }

  getFielList = async () => {
    const data = await request(`/pnode/getUrlAndToken?fileid=${wpsFileId.writer}`);
    this.initWap(data.wpsUrl || '');
  }

  initWap = async (wpsUrl: string) => {
    const wps = new window.WebOfficeSDK({
      mount: document.getElementById(`atContainer_${this.time}`),
      wpsUrl: wpsUrl,
      commonOptions: {
        isShowTopArea: false,
        isShowHeader: false,
        isIframeViewFullscreen: true
      },
      subscriptions: {
        'open.result': (e, data) => {
          this.setState({
            fileInfo: data.fileInfo
          }, () => {
            this.fetchUserInfo();
          });
        }
      }
    })
    this.wpsInstantce = wps;
    await wps.setToken({token: '123'}) // 设置token
    await wps.advancedApiReady();
    window.scrollTo(0, 0); // 避免存在多个文档时scroll滚动到底部
    const { loadSuccess } = this.props;
    loadSuccess && loadSuccess();
  }

  messageListener = (e) => {
    const { eventName } = JSON.parse(e.data);
    if (eventName === 'notice_message_at_info') {
      this.fecthMessageList();
    }
    if (eventName === 'at_init') {
      // 消息通知插件当前登录用户
      const { user } = this.state;
      this.wpsInstantce.iframe.contentWindow.postMessage(
        JSON.stringify({type: 'at_user_info', user: {userId: user.id, name: user.name}}), 
        this.wpsInstantce.origin
      )
    }

  }

  fetchUserInfo = () => {
    request(`/api/fetchUserInfo`).then(data => {
      this.setState({
        user: data.userInfo || {},
        wsConfig: data.wsConfig || {}
      }, () => {
        this.initWebSocket()
        this.fecthMessageList();
      })
    })
  }

  fecthMessageList = () => {
    const { user, fileInfo } = this.state;
    request(`/api/fetchMessageList?fileId=${fileInfo.id}&userId=${user.id}`).then(data => {
      this.setState({
        messageList: data
      })
    })
  }

  initWebSocket = () => {
    const { user, wsConfig } = this.state;
    const prefix = location.protocol === 'https:' ? 'wss' : 'ws';
    const host = location.protocol === 'https:' ? location.host : `${location.hostname}:${wsConfig.PORT}`
    this.wsServer = new WebSocket(`${prefix}://${host}/scene/at?userId=${user.id}`);
    this.wsServer.onopen = () => {
      this.wsServer.send('success');
      this.socketTimer = setInterval(() => {
        this.wsServer.send('ping');
      }, 60 * 1000)
    }
    this.wsServer.onmessage = (e: MessageEvent) => {
      try {
        const message = e.data;
        if (message === 'refresh') {
          this.fecthMessageList();
        }
      } catch (e) {
        console.error(e);
      }
    }
    this.wsServer.onclose = (e: CloseEvent) => {
      console.log("webscoket close", e);
      this.reconnect();
    }
    this.wsServer.onerror = (e: Event) => {
      console.log("webscoket error", e);
      this.reconnect();
    }
  }

  // webScoket断线重连
  reconnect = () => {
    this.socketTimer && clearInterval(this.socketTimer);
    this.reconnectTimer && clearTimeout(this.reconnectTimer);
    this.reconnectTimer = setTimeout(() => {
      this.initWebSocket();
    }, 30000)
  }

  onHandleScrollToAt = async (uuid: string) => {
    // 根据@的uuid跳转至指定位置
    const { fileInfo } = this.state;
    if (fileInfo.officeType !== 'w') return;

    const at = await this.wpsInstantce.Application.ActiveDocument.GetAtPos(uuid);
    if (at) {
      try {
        await this.wpsInstantce.Application.ActiveDocument.ActiveWindow.ScrollIntoView(at);
        // at.HighlightColorIndex = 14
        // 设置at高亮(临时闪烁)
        at.HighlightColorIndexTemp = 14
      }catch(e) {
        console.error(e)
      }
    }
  }

  render(): React.ReactNode {
    const { messageList, user } = this.state;
    const { style } = this.props;

    return (
      <div className={styles.atContainer} style={style}>
        <div className={styles.messageContainer}>
          <div className={styles.messageTitle}>
            <span className={styles.messageType}>全部</span>
            <span>用户名：{user.name}</span>
            {/* <img src={messageReadImg} className={styles.messageImg} title="消息列表"/> */}
          </div>
          <div className={styles.textContainer}>
            {
              (messageList || []).map((item) => {
                const Img = ImgObject[item.fileInfo.office_type]
                
                return (
                  <div key={item.user.uuid} className={styles.messageDiv} onClick={this.onHandleScrollToAt.bind(this, item.user.uuid)}>
                    <img src={Img} className={styles.officeImg}/>
                    <div className={styles.messageInfo}>
                      <div className={styles.fileInfo}>
                        {item.fileInfo.name}
                        <span className={styles.createTime}>{formatTime(item.create_time, 'MM-dd')}</span>
                      </div>
                      <div className={styles.message}>
                        <span className={styles.operator}>{item.creater.name}</span>
                        <span className={styles.gray}>在文档中提到了你</span>
                      </div>
                    </div>
                  </div>
                )
              })
            }
          </div>
        </div>
        <div id={`atContainer_${this.time}`} className={styles.wpsContainer}></div>
      </div>
    )
  }
}
