import React from 'react';
import { message } from 'antd';
import request from '../../../../utils/request';
import { initActiviteClass } from '../../../../utils/util';
import { isInViewPort } from '../../../../utils/dom';
import styles from './index.module.less';

interface SpeakerProps {
  url: string, // 文档地址
  meetId: string, // 会议ID
  loadSuccess: () => void, // 加载成功回调
}

let connectMessage = null; //socket重连提示

export default class Speaker extends React.Component<SpeakerProps> {

  wsServer = null;
  activiteObject = null;
  socketTimer = null;
  reconnectTimer = null;
  wpsInstantce = null;
  state = {
    user: null,
  }

  componentDidMount() {
    const { url } = this.props;
    if (url) {
      this.init();
    }
  }

  componentDidUpdate(preProps: SpeakerProps) {
    const { url, meetId } = this.props;
    if (preProps.url !== url && url) {
      // 通知观众切换文档
      const { user } = this.state;
      this.wsServer.send(JSON.stringify({id: user.id, meetId, data: {type: 'change_doc_url', value: url}}));

      this.restartWpsApi();
    }
  }

  componentWillUnmount() {
    this.socketTimer && clearInterval(this.socketTimer);
    this.reconnectTimer && clearTimeout(this.reconnectTimer);
  }

  init = async () => {
    const data = await request("/api/startMeeting");
    this.setState({
      user: data
    }, async () => {
      await this.initWap();
      this.initWsSocket();
    })
  }

  // 重新实例化新的wpsApi
  async restartWpsApi() {
    const wpsInstantce = this.wpsInstantce;
    if (wpsInstantce) {
      try {
        wpsInstantce.destroy()
      } catch(e) {
        console.error(e)
      }
    }

    await this.initWap()
    this.initWsSocket();
  }

  initWap = async () => {
    const { url } = this.props;
    const wps = new window.WebOfficeSDK({
      mount: document.getElementById('spakerContent'),
      wpsUrl: url,
      commonOptions: {
        // isShowTopArea: false,
        // isShowHeader: false,
        isIframeViewFullscreen: true
      },
      cooperUserAttribute: {
        isCooperUsersAvatarVisible: false
      }
    })
    await wps.setToken({token: '123'}) // 设置token
    this.onLoadSuccess();
    await wps.advancedApiReady();
    this.wpsInstantce = wps;
  }

  // 通过判断元素是否在可视区域内，调用success回调，避免页面阻塞
  onLoadSuccess = () => {
    const { loadSuccess } = this.props;
    const dom = document.getElementById('spakerContent');
    const isInView = isInViewPort(dom);
    if (!isInView) {
      loadSuccess && loadSuccess();
    }
  }

  initWsSocket = async () => {
    if (!this.wsServer) {
      this.webSocketConnect(() => {
        this.onWebSocketCallback();
      });
    } else {
      this.onWebSocketCallback();
    }
  }

  webSocketConnect = (wsOpenCallback: () => void) => {
    const { user } = this.state;
    const { meetId } = this.props;
    const { TYPE, PORT } = user.wsConfig;
    const host = window.location.host;
    const ip = host.split(':')[0];
    let wsUrl = '';
    if (PORT) {
      wsUrl = `${TYPE}://${ip}:${PORT}/scene/web?type=speaker&meetId=${meetId}&id=${user.id}`;
    } else {
      wsUrl = `${TYPE}://${ip}/scene/web?type=speaker&meetId=${meetId}&id=${user.id}`;
    }

    this.wsServer = new WebSocket(wsUrl);
    this.wsServer.onopen = async () => {
      wsOpenCallback && wsOpenCallback();
      this.socketTimer = setInterval(() => {
        this.wsServer.send('ping');
      }, 60 * 1000)
    }
    this.wsServer.onmessage = () => {
      // 收到信息后告知服务器，当前文档的最新动态
      this.activiteObject.sendInitInfo();
    }
    this.wsServer.onclose = (e: CloseEvent) => {
      console.log("webscoket close", e);
      this.reconnect();
    }
    this.wsServer.onerror = (e: Event) => {
      console.log("webscoket error", e);
      this.reconnect();
    }
  }

  // 连接成功后同步演讲者信息
  onWebSocketCallback = async () => {
    const { user } = this.state;
    const { meetId, loadSuccess } = this.props;
    const activiteClass = initActiviteClass(this.wpsInstantce);
    this.activiteObject = new activiteClass(this.wsServer, {meetId, user}, this.wpsInstantce.Application);
    this.activiteObject.speakerInit();
    await this.activiteObject.sendInitInfo(true);
    loadSuccess();
  }

  // webScoket断线重连
  reconnect = () => {
    if (!connectMessage) {
      connectMessage = true;
      message.error({content: '连接异常，请稍后重试！', key: 'wsMessage'});
    }

    this.socketTimer && clearInterval(this.socketTimer);
    this.reconnectTimer && clearTimeout(this.reconnectTimer);
    this.reconnectTimer = setTimeout(() => {
      this.webSocketConnect(() => {
        this.activiteObject.wsServer = this.wsServer;
        connectMessage = null;
        console.log('ws重连成功!');
      });
    }, 10000)
  }

  createCustomEvent = (e: CustomEvent) => {
    const myEvent = new CustomEvent('scrollStateChange', {detail: {show: e.type !== 'mouseenter'}});
    window.dispatchEvent(myEvent)
  }

  render(): React.ReactNode {
    return (
      <div id="spakerContent" className={styles.spakerContent}>
      </div>
    )
  }
}
