import { Request, Response } from 'express';
import { connection } from 'websocket';
import uuidv4 from 'uuid/v4';
import { getRandomChineseWord } from '../util';
import { queryDBData, saveDBData, deleteDBData, queryDBIndex } from '../util/db';


interface IUser {
  id: string,
  uuid?: string,
  name: string,
  status: boolean,
  avatar_url: string
}

interface IFileInfo {
  [propertys: string]: any,
  id: string,
}

interface IMessage {
  user: IUser,
  fileInfo: IFileInfo,
  message: string,
  status: boolean
}

interface IConnInfo {
  [key: string]: connection
}

let webUsers: IConnInfo = {}; // websocket用户信息;

export const At = {
  initialize(params: Record<string, any>, conn: connection): void {
    const { userId } = params;
    webUsers[userId] = conn;
    // 连接成功修改用户状态
    this.changeUserStatus(params, true);
  },
  onMessage(messageObject: Record<string, any>): void {
    let meetId = messageObject || '';
  },
  closeUserWs(params: Record<string, any>): void {
    this.changeUserStatus(params, false);
  },
  changeUserStatus(params: Record<string, any>, status: boolean): void {
    const { userId } = params;
    const { index, data } = queryDBIndex('/users', 'id', userId)

    if (index >= 0) {
      saveDBData(`/users[${index}]`, {...data, status});
    }
  }
}

function mockUsersList(req: Request): IUser[] {
  let name = <string>req.query.name || '';
  let status = req.query.status || 'true';
  const users = queryDBData('/users');
  const usersList = users.filter(item => {
    if (item.name.indexOf(name) >= 0 && `${item.status}` === status) {
      item.uuid = uuidv4();
      return true;
    }

    return false;
  })

  // mock分页查询展示
  return usersList.slice(0, 3); 
}


function mockUsersInfo(req: Request): IUser[] | null {
  let uuid = <string>req.query.uuid || '';
  let usersInfo: any = null;
  const messageList = queryDBData('/message');
  
  (messageList || []).forEach(( item: IMessage )  => {
    if (item.user.uuid === uuid) {
      usersInfo = item.user;
    }
  })

  return usersInfo;
}

function getUsersInfo(): IUser {
  const users = queryDBData('/users');
  let user: any = null;
  let userIndex = -1;
  (users || []).forEach((item, index) => {
    if (!item.status) {
      user = item;
      userIndex = index;
    }
  })

  if (!user) {
    user = {id: uuidv4(), name: getRandomChineseWord(), avatar_url: '', status: true};
    saveDBData('/users[]', user);
  } else {
    user = {...user, ...{status: true}}
    saveDBData(`/users[${userIndex}]`, user);
  }

  return user;
}

function updateUser(req: Request): void {
  const user = req.body.user && JSON.parse(req.body.user) || {};
  const isDelete = req.body.delete;
  const { index, data } = queryDBIndex('/users', 'id', user.id)
  if (isDelete) {
    deleteDBData(`/users[${index}]`);
  } else {
    if (index >= 0) {
      saveDBData(`/users[${index}]`, {...data, status: user.status});
    }
  }
}

function fetchMessages(req: Request): IMessage[] {
  let fileId = <string>req.query.fileId || '';
  let userId = <string>req.query.userId || '';
  const messageList = queryDBData('/message');
  const result = (messageList || []).filter((item: IMessage) => {
    return item.fileInfo.id === fileId && item.user.id === userId && item.status;
  })

  return result;
}

function saveMessageInfo(req: Request): void {
  const time = new Date().getTime();
  saveDBData('/message[]', {...req.body, create_time: time, status: true});

  // 推送消息给客户端
  const { user } = req.body;
  const conn = webUsers[user.id];
  if (conn) {
    conn.sendUTF('refresh');
  }
}

function deleteMessageInfo(req: Request): void {
  let fileId = <string>req.query.fileId || '';
  let uuid = <string>req.query.uuid || '';

  const messageList = queryDBData('/message');

  let index: Array<number> = [];
  messageList.forEach((item: any, fileIndex: number) => {
    if (item.fileInfo.id === fileId) {
      if (uuid) {
        if (uuid === item.user.uuid) {
          index.push(fileIndex);
        }
      } else {
        index.push(fileIndex);
      }
    }
  })

  // 删除message数据
  index.reverse().forEach((fileIndex) => {
    deleteDBData(`/message[${fileIndex}]`);
  })
}

function updateMessageStatus(req: Request): void {
  let uuid = <string>req.query.uuid || '';
  const messageList = queryDBData('/message');
  let messageIndex = -1;
  let filterMessage = (messageList || []).filter((item: IMessage, index) => {
    if (item.user.uuid === uuid) {
      messageIndex = index;
      return true;
    }

    return false;
  })
  
  if (messageIndex >= 0) {
    const message: IMessage = filterMessage[0];
    message.status = !message.status;
    saveDBData(`/message[${messageIndex}]`, message);
  }
}

export {
  mockUsersList,
  getUsersInfo,
  mockUsersInfo,
  saveMessageInfo,
  deleteMessageInfo,
  fetchMessages,
  updateMessageStatus,
  updateUser
}